package cz.muni.pa165.race;


import cz.muni.pa165.common_library.client.ClientConfig;
import cz.muni.pa165.common_library.exception.RestExceptionHandler;
import cz.muni.pa165.common_library.knot.CommonDbGetter;
import cz.muni.pa165.race.data.model.Race;
import cz.muni.pa165.race.data.model.Season;
import cz.muni.pa165.race.data.model.User;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.autoconfigure.domain.EntityScan;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Import;
import org.springframework.http.HttpMethod;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.http.SessionCreationPolicy;
import org.springframework.security.web.SecurityFilterChain;
import org.springframework.transaction.annotation.EnableTransactionManagement;

/**
 * Main app.
 */
@SpringBootApplication
@EnableWebSecurity
@EnableTransactionManagement
@EntityScan(basePackageClasses = {Race.class, Season.class, User.class})
@Import({RestExceptionHandler.class, ClientConfig.class, CommonDbGetter.class})
public class RaceApp {

  public static void main(String[] args) {
    SpringApplication.run(RaceApp.class, args);
  }

  @Autowired
  ApiKeyAuthManager apiKeyAuthManager;

  @Bean
  public SecurityFilterChain securityFilterChain(HttpSecurity httpSecurity) throws Exception {
    final SecurityFilter filter = new SecurityFilter();
    filter.setAuthenticationManager(apiKeyAuthManager);
    httpSecurity
        .csrf().disable()
        .sessionManagement().sessionCreationPolicy(SessionCreationPolicy.STATELESS)
        .and()
        .addFilter(filter)
        .authorizeHttpRequests()
        .requestMatchers(HttpMethod.GET, "/race/").authenticated()
        .anyRequest().permitAll();

    return httpSecurity.build();
  }
}
